==> gq-0.1.8.tar.gz: gq-0.1.8/query.c <==
#include <gtk/gtk.h>

#include <lber.h>
#include <ldap.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "configfile.h"
#include "query.h"
#include "detail.h"
#include "errorchain.h"
#include "util.h"
#include "debug.h"


struct resultset *cur_resultset = NULL;
struct attrs *cur_attrs = NULL;

extern GtkWidget *statusbar;
extern GtkWidget *main_clist;
extern struct gq_config config;


void free_resultset(struct resultset *set)
{
     struct resultset *nextset;

     while(set) {
	  nextset = set->next;
	  FREE(set, "struct resultset");
	  set = nextset;
     }

}


struct resultset *new_resultset(void)
{
     struct resultset *newset, *set;

     newset = MALLOC(sizeof(struct resultset), "struct resultset");
     newset->objectclass = NULL;
     newset->attributes = NULL;
     newset->num_attributes = 0;
     newset->next = NULL;

     if(!cur_resultset)
	  cur_resultset = newset;
     else {
	  for(set = cur_resultset; set->next; set = set->next)
	       ;
	  set->next = newset;
     }

     return(newset);
}


void statusbar_msg(char *message)
{
     static guint context, msgid = 0;

     if(!context)
	  context = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
						 "mainwin");
     if(msgid)
	  gtk_statusbar_remove(GTK_STATUSBAR(statusbar), context, msgid);

     gtk_statusbar_push(GTK_STATUSBAR(statusbar), context, message);

     /* make sure statusbar gets updated right away */
     while(gtk_events_pending())
	  gtk_main_iteration();

}


struct attrs *find_attr(char *attr)
{
     struct attrs *attrlist;

     attrlist = cur_attrs;
     while(attrlist) {
	  if(!strcmp(attrlist->name, attr))
	       return(attrlist);
	  attrlist = attrlist->next;
     }

     return(NULL);
}


void new_attr(char *attr)
{
     struct attrs *new_attr, *attrlist;

     new_attr = MALLOC(sizeof(struct attrs), "struct attrs");
     strncpy(new_attr->name, attr, MAX_ATTR_LEN);
     new_attr->next = NULL;

     if(cur_attrs) {
	  attrlist = cur_attrs;
	  while(attrlist->next)
	       attrlist = attrlist->next;
	  attrlist->next = new_attr;
     }
     else
	  cur_attrs = new_attr;

}


void store_attr(char *attr)
{
     struct attrs *attrlist;

     attrlist = find_attr(attr);

     if(!attrlist)
	  new_attr(attr);

}


void free_cur_attrs(void)
{
     struct attrs *next_attrs;

     while(cur_attrs) {
	  next_attrs = cur_attrs->next;;
	  FREE(cur_attrs, "struct attrs");
	  cur_attrs = next_attrs;
     }

}


struct ldapserver *server_by_name(char *name)
{
     struct ldapserver *server;

     server = config.ldapservers;
     while(server) {
	  if(!strcmp(server->name, name))
	       return(server);
	  server = server->next;
     }

     error_popup("Error finding server", "Where did it go?");

     return(NULL);
}


void query(char *querystring)
{
     LDAP *ld;
     LDAPMessage *res, *e;
     BerElement *ptr;

     GtkWidget *label, *new_main_clist, *parent_box;

     struct ldapserver *server;
     struct resultset *set;
     struct attrs *attrlist;
     gchar *cl[MAX_NUM_ATTRIBUTES];
     char tolist[MAX_NUM_ATTRIBUTES][128], message[128], filter[1024];
     char *attr, *dn, **vals;
     int msg, i, numentries, row;
     int attrcount, cur_col, col_offset;

     server = server_by_name(config.cur_servername);
     if(!server) {
	  statusbar_msg("Oops! No server found!");
	  return;
     }

     if(config.searchtype == ST_FILTER && querystring[0] == 0) {
	  statusbar_msg("Please enter a valid search filter");
	  return;
     }

     /* put this up now in case ldap_open() takes a while */
     sprintf(message, "Contacting %s...", server->ldaphost);
     statusbar_msg(message);

     if( (ld = open_connection(server)) == NULL)
	  return;

     /* construct filter */
     switch(config.searchtype) {
     case ST_SEARCH:
	  switch(config.search_argument) {
	  case SEARCHARG_BEGINS_WITH:
	       sprintf(filter, "(%s=%s*)", server->searchattr, querystring);
	       break;
	  case SEARCHARG_ENDS_WITH:
	       sprintf(filter, "(%s=*%s)", server->searchattr, querystring);
	       break;
	  case SEARCHARG_CONTAINS:
	       sprintf(filter, "(%s=*%s*)", server->searchattr, querystring);
	       break;
	  case SEARCHARG_EQUALS:
	       sprintf(filter, "(%s=%s)", server->searchattr, querystring);
	       break;

	  };
	  break;
     case ST_FILTER:
	  strncpy(filter, querystring, 1024);
	  break;
     }

     msg = ldap_search_s(ld, server->basedn, LDAP_SCOPE_SUBTREE,
			 filter, NULL, 0, &res);
     if(msg != LDAP_SUCCESS) {
	  statusbar_msg(ldap_err2string(msg));
	  return;
     }

     numentries = ldap_count_entries(ld, res);
     switch(numentries) {
     case 0:
	  sprintf(message, "No entries found");
	  break;
     case 1:
	  sprintf(message, "1 entry found");
	  break;
     default:
	  sprintf(message, "%d entries found", numentries);
     }
     statusbar_msg(message);

     if(numentries == 0)
	  return;

     free_resultset(cur_resultset);
     cur_resultset = NULL;

     /* dry run to collect attributes */
     for(e = ldap_first_entry(ld, res); e != NULL; e=ldap_next_entry(ld, e)) {

	  for(attr = ldap_first_attribute(ld, e, &ptr); attr != NULL;
	      attr = ldap_next_attribute(ld, e, ptr)) {

	       if(!strcasecmp(attr, "objectclass")) {
		    if(config.showoc)
			 store_attr(attr);
	       }
	       else
		    store_attr(attr);

	  }

     }

     /* figure out number of attributes to display */
     attrcount = 0;
     attrlist = cur_attrs;
     while(attrlist) {
	  attrcount++;
	  attrlist = attrlist->next;
     }

     /* build new clist */
     new_main_clist = gtk_clist_new(attrcount + (config.showdn ? 1 : 0) );
     gtk_container_border_width(GTK_CONTAINER(new_main_clist), 2);
     gtk_clist_column_titles_show(GTK_CLIST(new_main_clist));
     gtk_clist_set_policy(GTK_CLIST(new_main_clist), GTK_POLICY_AUTOMATIC,
                          GTK_POLICY_AUTOMATIC);
     gtk_signal_connect(GTK_OBJECT(new_main_clist), "select_row",
                        GTK_SIGNAL_FUNC(select_entry_callback),
                        NULL);

     /* DN goes in first column */
     col_offset = 0;
     if(config.showdn) {
	  label = gtk_label_new("DN");
	  gtk_widget_show(label);
	  gtk_clist_set_column_widget(GTK_CLIST(new_main_clist), 0, label);
	  gtk_clist_set_column_width(GTK_CLIST(new_main_clist), 0, 260);
	  col_offset++;
     }



     /* set up titles */
     attrlist = cur_attrs;
     for(i = 0; i < attrcount; i++) {
	  attrlist->column = col_offset + i;
	  label = gtk_label_new(attrlist->name);
	  gtk_widget_show(label);
	  gtk_clist_set_column_widget(GTK_CLIST(new_main_clist),
				      attrlist->column, label);
	  gtk_clist_set_column_width(GTK_CLIST(new_main_clist),
				     attrlist->column, 120);
	  attrlist = attrlist->next;
     }

     row = 0;
     for(e = ldap_first_entry(ld, res); e != NULL; e=ldap_next_entry(ld, e)) {

	  /* not every attribute necessarily comes back for every entry,
	   * so a clear this every time */
	  for(i = 0; i < MAX_NUM_ATTRIBUTES; i++) {
	       cl[i] = NULL;
	       tolist[i][0] = '\0';
	  }

	  dn = ldap_get_dn(ld, e);

	  /* store for later reference */
	  set = new_resultset();
	  strncpy(set->dn, dn, MAX_DN_LEN);

	  if(config.showdn) {
	       strcpy(tolist[0],dn);
	       cl[0] = tolist[0];
	  }

	  free(dn);

	  for(attr = ldap_first_attribute(ld, e, &ptr); attr != NULL;
	      attr = ldap_next_attribute(ld, e, ptr)) {

	       attrlist = find_attr(attr);
	       if(attrlist) {
		    cur_col = attrlist->column;

		    vals = ldap_get_values(ld, e, attr);
		    if(vals) {
			 for(i = 0; vals[i] != NULL; i++) {
			      if(i == 0)
				   strcpy(tolist[cur_col], vals[i]);
			      else {
				   strcat(tolist[cur_col], " ");
				   strcat(tolist[cur_col], vals[i]);
			      }

			 }
			 ldap_value_free(vals);
			 cl[cur_col] = tolist[cur_col];
		    }

	       }

	  }

	  /* insert row into result window */
	  gtk_clist_append(GTK_CLIST(new_main_clist), cl);
	  gtk_clist_set_row_data(GTK_CLIST(new_main_clist), row, set);
	  row++;
     }
     free_cur_attrs();

     parent_box = main_clist->parent;
     gtk_clist_clear(GTK_CLIST(main_clist));
     gtk_widget_destroy(main_clist);
     gtk_box_pack_start(GTK_BOX(parent_box), new_main_clist, TRUE, TRUE, 0);
     gtk_widget_show(new_main_clist);
     main_clist = new_main_clist;

     ldap_msgfree(res);
}


void results_popup_menu(GdkEventButton *event, struct resultset *set)
{
     GtkWidget *root_menu, *menu, *menu_item;

     root_menu = gtk_menu_item_new_with_label("Root");
     gtk_widget_show(root_menu);

     menu = gtk_menu_new();
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(root_menu), menu);

     /* View */
     menu_item = gtk_menu_item_new_with_label("View");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(view_entry),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     /* Edit */
     menu_item = gtk_menu_item_new_with_label("Edit");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(edit_entry_values),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     /* Use as template */
     menu_item = gtk_menu_item_new_with_label("Use as template");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(edit_entry_novalues),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     /* separator */
     menu_item = gtk_menu_item_new();
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_widget_show(menu_item);

     /* Delete */
     menu_item = gtk_menu_item_new_with_label("Delete");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(delete_entry),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
		    event->button, event->time);

}


int select_entry_callback(GtkWidget *clist, gint row, gint column,
			   GdkEventButton *event, gpointer data)
{
     struct resultset *set;

     set = gtk_clist_get_row_data(GTK_CLIST(clist), row);

     if(event->button == 1 && event->type == GDK_2BUTTON_PRESS)
	  edit_entry_values(set);
     else if(event->button == 3)
	  results_popup_menu(event, set);

     return(TRUE);
}

==> gq-0.1.9.tar.gz: gq-0.1.9/query.c <==
#include <gtk/gtk.h>

#include <lber.h>
#include <ldap.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "configfile.h"
#include "query.h"
#include "detail.h"
#include "errorchain.h"
#include "util.h"
#include "debug.h"


struct resultset *cur_resultset = NULL;
struct attrs *cur_attrs = NULL;

extern GtkWidget *statusbar;
extern GtkWidget *main_clist;
extern struct gq_config config;


void free_resultset(struct resultset *set)
{
     struct resultset *nextset;

     while(set) {
	  nextset = set->next;
	  FREE(set, "struct resultset");
	  set = nextset;
     }

}


struct resultset *new_resultset(void)
{
     struct resultset *newset, *set;

     newset = MALLOC(sizeof(struct resultset), "struct resultset");
     newset->objectclass = NULL;
     newset->attributes = NULL;
     newset->num_attributes = 0;
     newset->next = NULL;

     if(!cur_resultset)
	  cur_resultset = newset;
     else {
	  for(set = cur_resultset; set->next; set = set->next)
	       ;
	  set->next = newset;
     }

     return(newset);
}


void statusbar_msg(char *message)
{
     static guint context, msgid = 0;

     if(!context)
	  context = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
						 "mainwin");
     if(msgid)
	  gtk_statusbar_remove(GTK_STATUSBAR(statusbar), context, msgid);

     gtk_statusbar_push(GTK_STATUSBAR(statusbar), context, message);

     /* make sure statusbar gets updated right away */
     while(gtk_events_pending())
	  gtk_main_iteration();

}


int new_attr(char *attr)
{
     struct attrs *new_attr, *attrlist;
     int cnt;

     new_attr = MALLOC(sizeof(struct attrs), "struct attrs");
     strncpy(new_attr->name, attr, MAX_ATTR_LEN);
     new_attr->next = NULL;

     cnt = 0;
     if(cur_attrs) {
	  cnt++;
	  attrlist = cur_attrs;
	  while(attrlist->next) {
	       attrlist = attrlist->next;
	       cnt++;
	  }
	  attrlist->next = new_attr;
     }
     else
	  cur_attrs = new_attr;

     new_attr->column = cnt;

     return(cnt);
}

struct attrs *find_attr(char *attr)
{
     struct attrs *attrlist;

     attrlist = cur_attrs;
     while(attrlist) {
	  if(!strcasecmp(attrlist->name, attr))
	       return(attrlist);
	  attrlist = attrlist->next;
     }

     return(NULL);
}

int column_by_attr(char *attribute)
{
     struct attrs *attr;

     attr = find_attr(attribute);
     if(!attr)
	  return(new_attr(attribute));
     else
	  return(attr->column);

}

void free_cur_attrs(void)
{
     struct attrs *next_attrs;

     while(cur_attrs) {
	  next_attrs = cur_attrs->next;;
	  FREE(cur_attrs, "struct attrs");
	  cur_attrs = next_attrs;
     }

}


struct ldapserver *server_by_name(char *name)
{
     struct ldapserver *server;

     if(name == NULL)
	  return(NULL);

     server = config.ldapservers;
     while(server) {
	  if(!strcmp(server->name, name))
	       return(server);
	  server = server->next;
     }

     error_popup("Error finding server", "Where did it go?");

     return(NULL);
}


void query(char *querystring)
{
     LDAP *ld;
     LDAPMessage *res, *e;
     BerElement *ptr;
     GtkWidget *new_main_clist, *scrwin;
     struct ldapserver *server;
     struct resultset *set;
     gchar *cl[MAX_NUM_ATTRIBUTES];
     char tolist[MAX_NUM_ATTRIBUTES][128], message[128], filter[1024];
     char *attr, *dn, **vals;
     int msg, rc, i, row;
     int cur_col, oc_col, columns_done[MAX_NUM_ATTRIBUTES];

     for(i = 0; i < MAX_NUM_ATTRIBUTES; i++)
	  columns_done[i] = 0;

     server = server_by_name(config.cur_servername);
     if(!server) {
	  statusbar_msg("Oops! No server found!");
	  return;
     }

     if(config.searchtype == ST_FILTER && querystring[0] == 0) {
	  statusbar_msg("Please enter a valid search filter");
	  return;
     }

     /* put this up now in case ldap_open() takes a while */
     sprintf(message, "Contacting %s...", server->ldaphost);
     statusbar_msg(message);

     if( (ld = open_connection(server)) == NULL)
	  return;

     /* construct filter */
     switch(config.searchtype) {
     case ST_SEARCH:
	  switch(config.search_argument) {
	  case SEARCHARG_BEGINS_WITH:
	       sprintf(filter, "(%s=%s*)", server->searchattr, querystring);
	       break;
	  case SEARCHARG_ENDS_WITH:
	       sprintf(filter, "(%s=*%s)", server->searchattr, querystring);
	       break;
	  case SEARCHARG_CONTAINS:
	       sprintf(filter, "(%s=*%s*)", server->searchattr, querystring);
	       break;
	  case SEARCHARG_EQUALS:
	       sprintf(filter, "(%s=%s)", server->searchattr, querystring);
	       break;

	  };
	  break;
     case ST_FILTER:
	  strncpy(filter, querystring, 1024);
	  break;
     };

     statusbar_msg("searching...");
     msg = ldap_search(ld, server->basedn, LDAP_SCOPE_SUBTREE,
			 filter, NULL, 0);

     if(msg == -1) {
	  statusbar_msg(ldap_err2string(msg));
	  return;
     }

     free_resultset(cur_resultset);
     cur_resultset = NULL;

     /* build new clist */
     new_main_clist = gtk_clist_new(MAX_NUM_ATTRIBUTES);
     GTK_CLIST(new_main_clist)->button_actions[2] = GTK_BUTTON_SELECTS;
     GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(new_main_clist), GTK_CAN_FOCUS);
     gtk_widget_show(new_main_clist);
     gtk_clist_column_titles_show(GTK_CLIST(new_main_clist));
     gtk_signal_connect(GTK_OBJECT(new_main_clist), "select_row",
                        GTK_SIGNAL_FUNC(select_entry_callback),
                        NULL);

     scrwin = gtk_object_get_data(GTK_OBJECT(main_clist), "scrwin");
     gtk_object_set_data(GTK_OBJECT(new_main_clist), "scrwin", scrwin);
     gtk_clist_clear(GTK_CLIST(main_clist));
     gtk_widget_destroy(main_clist);
     main_clist = new_main_clist;
     gtk_container_add(GTK_CONTAINER(scrwin), new_main_clist);

     /* reserve columns 0 & 1 for DN and objectClass, respectively */
     if(config.showdn) {
	  column_by_attr("DN");
	  gtk_clist_set_column_title(GTK_CLIST(new_main_clist), 0, "DN");
	  gtk_clist_set_column_width(GTK_CLIST(new_main_clist), 0, 260);
     }
     oc_col = column_by_attr("objectClass");
     gtk_clist_set_column_title(GTK_CLIST(new_main_clist), oc_col, "objectClass");
     gtk_clist_set_column_width(GTK_CLIST(new_main_clist), oc_col, 120);
     columns_done[oc_col] = 1;
     if(config.showoc == 0)
	  gtk_clist_set_column_visibility(GTK_CLIST(new_main_clist), oc_col, 0);


     row = 0;
     while( (rc = ldap_result(ld, msg, 0, NULL, &res)) == LDAP_RES_SEARCH_ENTRY) {
	  for(e = ldap_first_entry(ld, res); e != NULL; e=ldap_next_entry(ld, e)) {

	       /* not every attribute necessarily comes back for every entry,
		* so clear this every time */
	       for(i = 0; i < MAX_NUM_ATTRIBUTES; i++) {
		    cl[i] = NULL;
		    tolist[i][0] = '\0';
	       }

	       dn = ldap_get_dn(ld, e);
	       /* store for later reference */
	       set = new_resultset();
	       strncpy(set->dn, dn, MAX_DN_LEN);

	       if(config.showdn) {
		    strcpy(tolist[0],dn);
		    cl[0] = tolist[0];
	       }
	       free(dn);

	       for(attr = ldap_first_attribute(ld, e, &ptr); attr != NULL;
		   attr = ldap_next_attribute(ld, e, ptr)) {

		    cur_col = column_by_attr(attr);
		    if(cur_col == MAX_NUM_ATTRIBUTES)
			 break;

		    if(!columns_done[cur_col]) {
			 gtk_clist_set_column_title(GTK_CLIST(new_main_clist), cur_col, attr);
			 gtk_clist_set_column_width(GTK_CLIST(new_main_clist), cur_col, 120);
			 columns_done[cur_col] = 1;
		    }

		    vals = ldap_get_values(ld, e, attr);
		    if(vals) {
			 for(i = 0; vals[i] != NULL; i++) {
			      if(i == 0)
				   strcpy(tolist[cur_col], vals[i]);
			      else {
				   strcat(tolist[cur_col], " ");
				   strcat(tolist[cur_col], vals[i]);
			      }

			 }
			 ldap_value_free(vals);
			 cl[cur_col] = tolist[cur_col];
		    }

	       }

	       /* insert row into result window */
	       gtk_clist_append(GTK_CLIST(new_main_clist), cl);
	       gtk_clist_set_row_data(GTK_CLIST(new_main_clist), row, set);
	       row++;
	  }

     }

     if(rc == -1)
	  statusbar_msg(ldap_err2string(msg));
     else {
	  switch(row) {
	  case 0:
	       sprintf(message, "No entries found");
	       break;
	  case 1:
	       sprintf(message, "1 entry found");
	       break;
	  default:
	       sprintf(message, "%d entries found", row);
	  }
	  statusbar_msg(message);
     }

     gtk_clist_column_titles_passive(GTK_CLIST(new_main_clist));
     free_cur_attrs();
     ldap_msgfree(res);

}


void results_popup_menu(GdkEventButton *event, struct resultset *set)
{
     GtkWidget *root_menu, *menu, *menu_item;

     root_menu = gtk_menu_item_new_with_label("Root");
     gtk_widget_show(root_menu);

     menu = gtk_menu_new();
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(root_menu), menu);

     /* View */
     menu_item = gtk_menu_item_new_with_label("View");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(view_entry),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     /* Edit */
     menu_item = gtk_menu_item_new_with_label("Edit");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(edit_entry_values),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     /* Use as template */
     menu_item = gtk_menu_item_new_with_label("Use as template");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(edit_entry_novalues),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     /* separator */
     menu_item = gtk_menu_item_new();
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_widget_show(menu_item);

     /* Delete */
     menu_item = gtk_menu_item_new_with_label("Delete");
     gtk_menu_append(GTK_MENU(menu), menu_item);
     gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			       GTK_SIGNAL_FUNC(delete_entry),
			       (gpointer) set);
     gtk_widget_show(menu_item);

     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
		    event->button, event->time);

}


int select_entry_callback(GtkWidget *clist, gint row, gint column,
			   GdkEventButton *event, gpointer data)
{
     struct resultset *set;

     if(event) {
	  set = gtk_clist_get_row_data(GTK_CLIST(clist), row);

	  if(event->button == 1 && event->type == GDK_2BUTTON_PRESS)
	       edit_entry_values(set);
	  else if(event->button == 3)
	       results_popup_menu(event, set);
     }

     return(TRUE);
}
